@@ -1,9 +1,8 @@
 package JIRA::REST;
 # ABSTRACT: Thin wrapper around Jira's REST API
-$JIRA::REST::VERSION = '0.020';
-use 5.010;
+$JIRA::REST::VERSION = '0.022';
+use 5.016;
 use utf8;
-use strict;
 use warnings;
 
 use Carp;
@@ -13,33 +12,10 @@ use MIME::Base64;
 use URI::Escape;
 use JSON 2.23;
 use REST::Client;
+use HTTP::CookieJar::LWP;
 
 sub new {
-    my $class = shift; # this always has to come first!
-
-    # Valid option names in the order expected by the old-form constructor
-    my @opts = qw/url username password rest_client_config proxy ssl_verify_none anonymous/;
-
-    my %args;
-
-    if (@_ == 1 && ref $_[0] && ref $_[0] eq 'HASH') {
-        # The new-form constructor expects a single hash reference.
-        @args{@opts} = delete @{$_[0]}{@opts};
-        croak __PACKAGE__ . "::new: unknown arguments: '", join("', '", sort keys %{$_[0]}), "'.\n"
-            if keys %{$_[0]};
-    } else {
-        # The old-form constructor expects a list of positional parameters.
-        @args{@opts} = @_;
-    }
-
-    # Turn the url into a URI object
-    if (! $args{url}) {
-        croak __PACKAGE__ . "::new: 'url' argument must be defined.\n";
-    } elsif (! ref $args{url}) {
-        $args{url} = URI->new($args{url});
-    } elsif (! $args{url}->isa('URI')) {
-        croak __PACKAGE__ . "::new: 'url' argument must be a URI object.\n";
-    }
+    my ($class, %args) = &_grok_args;
 
     my ($path, $api) = ($args{url}->path, '/rest/api/latest');
     # See if the user wants a default REST API:
@@ -53,7 +29,7 @@ sub new {
         $args{url}->path($path);
     }
 
-    unless ($args{anonymous}) {
+    unless ($args{anonymous} || $args{pat}) {
         # If username and password are not set we try to lookup the credentials
         if (! defined $args{username} || ! defined $args{password}) {
             ($args{username}, $args{password}) =
@@ -62,10 +38,88 @@ sub new {
 
         foreach (qw/username password/) {
             croak __PACKAGE__ . "::new: '$_' argument must be a non-empty string.\n"
-                unless defined $args{$_} && ! ref $args{$_} && length $args{$_};
+                if ! defined $args{$_} || ref $args{$_} || length $args{$_} == 0;
         }
     }
 
+    my $rest = REST::Client->new($args{rest_client_config});
+
+    # Set default base URL
+    $rest->setHost($args{url});
+
+    # Follow redirects/authentication by default
+    $rest->setFollow(1);
+
+    unless ($args{anonymous} || $args{session}) {
+        # Since Jira doesn't send an authentication challenge, we force the
+        # sending of the authentication header.
+        $rest->addHeader(
+            Authorization =>
+                $args{pat}
+                ? "Bearer $args{pat}"
+                : 'Basic ' . encode_base64("$args{username}:$args{password}", '')
+            );
+    }
+
+    for my $ua ($rest->getUseragent) {
+        # Configure UserAgent name
+        $ua->agent(__PACKAGE__);
+
+        # Set proxy to be used
+        $ua->proxy(['http','https'] => $args{proxy}) if $args{proxy};
+
+        # Turn off SSL verification if requested
+        $ua->ssl_opts(SSL_verify_mode => 0, verify_hostname => 0) if $args{ssl_verify_none};
+
+        # Configure a cookie_jar so that we can send Cookie headers
+        $ua->cookie_jar(HTTP::CookieJar::LWP->new());
+    }
+
+    my $jira = bless {
+        rest => $rest,
+        json => JSON->new->utf8->allow_nonref,
+        api  => $api,
+    } => $class;
+
+    $jira->{_session} = $jira->POST('/rest/auth/1/session', undef, {
+        username => $args{username},
+        password => $args{password},
+    }) if $args{session};
+
+    return $jira;
+}
+
+sub _grok_args {
+    my ($class, @args) = @_;
+
+    # Valid option names in the order expected by the old-form constructor
+    my @opts = qw/url username password rest_client_config proxy ssl_verify_none anonymous pat session/;
+
+    my %args;
+
+    if (@args == 1 && ref $args[0] && ref $args[0] eq 'HASH') {
+        # The new-form constructor expects a single hash reference.
+        @args{@opts} = delete @{$args[0]}{@opts};
+        croak __PACKAGE__ . "::new: unknown arguments: '", join("', '", sort keys %{$args[0]}), "'.\n"
+            if keys %{$args[0]};
+    } else {
+        # The old-form constructor expects a list of positional parameters.
+        @args{@opts} = @args;
+    }
+
+    # Turn the url into a URI object
+    if (! $args{url}) {
+        croak __PACKAGE__ . "::new: 'url' argument must be defined.\n";
+    } elsif (! ref $args{url}) {
+        $args{url} = URI->new($args{url});
+    } elsif (! $args{url}->isa('URI')) {
+        croak __PACKAGE__ . "::new: 'url' argument must be a URI object.\n";
+    }
+
+    if (!!$args{anonymous} + !!$args{pat} + !!$args{session} > 1) {
+        croak __PACKAGE__ . "::new: 'anonymous', 'pat', and 'session' are mutually exclusive options.\n"
+    }
+
     for ($args{rest_client_config}) {
         $_ //= {};
         croak __PACKAGE__ . "::new: 'rest_client_config' argument must be a hash reference.\n"
@@ -79,35 +133,25 @@ sub new {
         $args{proxy} //= $proxy;
     }
 
-    my $rest = REST::Client->new($args{rest_client_config});
+    return ($class, %args);
+}
 
-    # Set default base URL
-    $rest->setHost($args{url});
+sub new_session {
+    my ($class, @args) = @_;
 
-    # Follow redirects/authentication by default
-    $rest->setFollow(1);
-
-    # Since Jira doesn't send an authentication challenge, we force the
-    # sending of the authentication header.
-    $rest->addHeader(Authorization => 'Basic ' . encode_base64("$args{username}:$args{password}", ''))
-        unless $args{anonymous};
-
-    for my $ua ($rest->getUseragent) {
-        # Configure UserAgent name
-        $ua->agent(__PACKAGE__);
-
-        # Set proxy to be used
-        $ua->proxy(['http','https'] => $args{proxy}) if $args{proxy};
-
-        # Turn off SSL verification if requested
-        $ua->ssl_opts(SSL_verify_mode => 0, verify_hostname => 0) if $args{ssl_verify_none};
+    if (@args == 1 && ref $args[0] && ref $args[0] eq 'HASH') {
+        $args[0]{session} = 1;
+    } else {
+        $args[8] = 1;
     }
 
-    return bless {
-        rest => $rest,
-        json => JSON->new->utf8->allow_nonref,
-        api  => $api,
-    } => $class;
+    return $class->new(@args);
+}
+
+sub DESTROY {
+    my $self = shift;
+    $self->DELETE('/rest/auth/1/session') if exists $self->{_session};
+    return;
 }
 
 sub _search_for_credentials {
@@ -363,6 +407,8 @@ sub attach_files {
         $response->is_success
             or croak $self->_error("attach_files($file): " . $response->status_line);
     }
+
+    return;
 }
 
 1;
@@ -379,7 +425,7 @@ JIRA::REST - Thin wrapper around Jira's REST API
 
 =head1 VERSION
 
-version 0.020
+version 0.022
 
 =head1 SYNOPSIS
 
@@ -391,6 +437,23 @@ version 0.020
         password => 'mypass',
     });
 
+    my $jira_with_session = JIRA::REST->new({
+        url      => 'https://jira.example.net',
+        username => 'myuser',
+        password => 'mypass',
+        session  => 1,
+    });
+
+    my $jira_with_pat = JIRA::REST->new({
+        url => 'https://jira.example.net',
+        pat => 'NDc4NDkyNDg3ODE3OstHYSeYC1GnuqRacSqvUbookcZk',
+    });
+
+    my $jira_anonymous = JIRA::REST->new({
+        url => 'https://jira.example.net',
+        anonymous => 1,
+    });
+
     # File a bug
     my $issue = $jira->POST('/issue', undef, {
         fields => {
@@ -459,17 +522,17 @@ endpoints have a path prefix of C</rest/agile/VERSION>.
 
 =back
 
-=head1 CONSTRUCTOR
+=head1 CONSTRUCTORS
 
 =head2 new HASHREF
 
-=head2 new URL, USERNAME, PASSWORD, REST_CLIENT_CONFIG, ANONYMOUS, PROXY, SSL_VERIFY_NONE
+=head2 new URL, USERNAME, PASSWORD, REST_CLIENT_CONFIG, PROXY, SSL_VERIFY_NONE, ANONYMOUS, PAT, SESSION
 
-The constructor can take its arguments from a single hash reference or from
-a list of positional parameters. The first form is preferred because it lets
-you specify only the arguments you need. The second form forces you to pass
-undefined values if you need to pass a specific value to an argument further
-to the right.
+The default constructor can take its arguments from a single hash reference or
+from a list of positional parameters. The first form is preferred because it
+lets you specify only the arguments you need. The second form forces you to pass
+undefined values if you need to pass a specific value to an argument further to
+the right.
 
 The arguments are described below with the names which must be used as the
 hash keys:
@@ -508,9 +571,9 @@ during construction.
 
 The username and password of a Jira user to use for authentication.
 
-If B<anonymous> is false then, if either B<username> or B<password> isn't
-defined the module looks them up in either the C<.netrc> file or via
-L<Config::Identity> (which allows C<gpg> encrypted credentials).
+If B<anonymous> is false and no B<pat> given, then, if either B<username> or
+B<password> isn't defined the module looks them up in either the C<.netrc> file
+or via L<Config::Identity> (which allows C<gpg> encrypted credentials).
 
 L<Config::Identity> will look for F<~/.jira-identity> or F<~/.jira>.
 You can change the filename stub from C<jira> to a custom stub with the
@@ -543,12 +606,42 @@ pass L<LWP::UserAgent>'s verification methods.
 
 =item * B<anonymous>
 
-Tells the module that you want to connect to the specified Jira server with
-no username or password.  This way you can access public Jira servers
-without needing to authenticate.
+=item * B<pat>
+
+=item * B<session>
+
+These three arguments are mutually exclusive, i.e., you can use at most one of
+them. By default, they are all undefined.
+
+The boolean B<anonymous> argument tells the module if you want to connect to the
+specified Jira with no authentication. This allows you to get some information
+from open or public Jira servers. If enabled, the B<username> and B<password>
+arguments are disregarded.
+
+The B<pat> argument maps to a string which should be a personal access token
+that can be used for authentication instead of a username and a password.  This
+option is available since Jira version 8.14.  Please refer to
+L<https://confluence.atlassian.com/enterprise/using-personal-access-tokens-1026032365.html>
+for details. If enabled, the B<username> and B<password> arguments are
+disregarded.
+
+The booleal B<session> argument tells the module if you want it to acquire a
+session cookie by making a C<POST /rest/auth/1/session> call to login to
+Jira. This is particularly useful when interacting with Jira Data Center,
+because it can use the session cookie to maintain affinity with one of the
+redundant servers. Upon destruction, the object makes a C<DELETE
+/rest/auth/1/session> call to logout from Jira. If enabled, the B<username> and
+B<password> arguments are required.
 
 =back
 
+=head2 new_session OPTIONS
+
+This alternative constructor simply invokes the default constructor with the
+same options, adding to them the B<session> option. New code should use the
+default constructor with the B<session> option because this constructor may be
+deprecated in the future.
+
 =head1 REST METHODS
 
 Jira's REST API documentation lists dozens of "resources" which can be
@@ -695,22 +788,22 @@ interface to attach files to issues.
 
 =head1 PERL AND JIRA COMPATIBILITY POLICY
 
-Currently L<JIRA::REST> requires Perl 5.10 and supports Jira 7.0.
+Currently L<JIRA::REST> requires Perl 5.16 and supports Jira 7.0.
 
 We try to be compatible with the Perl native packages of the oldest L<Ubuntu
 LTS|https://www.ubuntu.com/info/release-end-of-life> and
 L<CentOS|https://wiki.centos.org/About/Product> Linux distributions still
 getting maintainance updates.
 
-  +-----------------------+------+-------------+
-  | Distro                | Perl | End of Life |
-  +-----------------------+------+-------------+
-  | Ubuntu 14.04 (trusty) | 5.18 |   2019-04   |
-  | Ubuntu 16.04 (xenial) | 5.22 |   2021-04   |
-  | Ubuntu 18.04 (bionic) | 5.26 |   2023-04   |
-  | CentOS 6              | 5.10 |   2020-12   |
-  | CentOS 7              | 5.16 |   2024-07   |
-  +-----------------------+------+-------------+
+  +-------------+-----------------------+------+
+  | End of Life | Distro                | Perl |
+  +-------------+-----------------------+------+
+  |   2021-04   | Ubuntu 16.04 (xenial) | 5.22 |
+  |   2023-04   | Ubuntu 18.04 (bionic) | 5.26 |
+  |   2024-07   | CentOS 7              | 5.16 |
+  |   2025-04   | Ubuntu 20.04 (focal ) | 5.30 |
+  |   2029-05   | CentOS 8              | 5.26 |
+  +-------------+-----------------------+------+
 
 As you can see, we're kept behind mostly by the slow pace of CentOS (actually,
 RHEL) releases.
@@ -751,7 +844,7 @@ Gustavo L. de M. Chaves <gnustavo@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2019 by CPqD <www.cpqd.com.br>.
+This software is copyright (c) 2021 by CPQD <www.cpqd.com.br>.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
